;**********************************************************
;*                  SER_LED.ASM  V1.0                     *
;* Ansteuerung eines vierstelligen LED-Displays           *
;* Mikrocontroller: AT89C2051, f=11,0592 MHz              *
;* Serielle Uebertragung der Anzeigedaten zum Mikro-      *
;* controller, Umwandlung in IC-Bus-Daten fuer den       *
;* LED-Displaytreiber SAA1064                             *
;* Ser. Format: 8N1, 9600 oder 1200 Baud, kein Handshake  *
;**********************************************************
;* World Of Electronics                                   *
;* http://www.woe.de.vu                                   *
;* (c) WOE, 1999                                          *
;**********************************************************

; P3.  7       5      3
;     SDA     SCL   Jumper


sda equ p3.7
scl equ p3.5
jumper equ p3.3        ; Jumper offen: 9600 Baud
                       ; Jumper an Masse: 1200 Baud


i2c_ack_in equ 0       ; vom Slave gesendetes Acknowledge-Bit
i2c_ack_out equ 1      ; an den Slave zu sendendes Acknowledge-Bit
                       ; jeweils 0=Ack, 1=kein Ack

nibble_valid equ 2     ; 1=gueltiges Nibble, 0=ungueltig
hex_valid    equ 3     ; 1=gueltiges Hexzeichen, 0=ungueltig
dec_valid    equ 4     ; 1=gueltiges Dezimalzeichen, 0=ungueltig

disp_mem equ 100       ; Basisadresse fuer Speicherplaetze der Displaydaten


       setb i2c_ack_out
       setb i2c_ack_in

       jnb jumper,b1200
b9600:
       mov th1, #0fdh   ; 9600 Baud
       mov tl1, #0fdh
       jmp b_end
b1200:
       mov th1, #0e8h   ; 1200 Baud
       mov tl1, #0e8h
b_end:
       mov tmod, #20h
       mov scon, #50h
       setb tr1
       mov a, sbuf
       clr ri
       call init_disp
       mov r5,#0

wdh:

wait_1:
       jnb ri,wait_1
       mov a,sbuf
       clr ri
       cjne a,#'%',l1     ;  % -> Binaermode
       jmp prg1
l1:
       cjne a,#'$',l2     ;  $ -> Hexmode
       jmp prg2
l2:
       cjne a,#'#',l3     ;  # -> Dezimalmode
       jmp prg3
l3:
       cjne a,#'?',l4     ;  ? -> Displaytest
       jmp prg4
l4:
       cjne a,#'/',l5     ;  / -> Display loeschen
       jmp prg5
l5:    
       cjne a,#'&',wdh    ;  & -> Helligkeit steuern
       jmp prg6



prg1:                        ; Binaermode: 8 Nibbles
       mov r6,#4
       mov r0,#disp_mem
p1_l1:

p1_w1:
       jnb ri,p1_w1
       mov a,sbuf
       clr ri
       call check_char
       jnb nibble_valid,p1_w1

       mov b,#16
       mul ab
       mov r7,a

p1_w2:
       jnb ri,p1_w2
       mov a,sbuf
       clr ri
       call check_char
       jnb nibble_valid,p1_w2

       add a,r7
       mov @r0,a
       inc r0
       djnz r6,p1_l1

       call send_disp
       jmp wdh


prg2:                        ; Hexmode: 4 x Hex-Zeichen
       mov r6,#4
       mov r0,#disp_mem
p2_l1:

p2_w1:
       jnb ri,p2_w1
       mov a,sbuf
       clr ri
       cjne a,#'.',p2_n1
       mov r5,#80h
       jmp p2_w1
p2_n1:
       call check_char
       jnb hex_valid,p2_w1

       mov dptr,#disp_tab
       movc a,@a+dptr       
       orl a,r5
       mov @r0,a
       inc r0
       mov r5,#0
       djnz r6,p2_l1

       call send_disp
       jmp wdh


prg3:                        ; Dezimalmode: 4 x Dezimal-Zeichen
       mov r6,#4
       mov r0,#disp_mem
p3_l1:

p3_w1:
       jnb ri,p3_w1
       mov a,sbuf
       clr ri
       cjne a,#'.',p3_n1
       mov r5,#80h
       jmp p3_w1
p3_n1:
       call check_char
       jnb dec_valid,p3_w1
       
       mov dptr,#disp_tab
       movc a,@a+dptr       
       orl a,r5
       mov @r0,a
       inc r0
       mov r5,#0
       djnz r6,p3_l1

       call send_disp
       jmp wdh


prg4:                        ; Displaytest: alle Segmente an
       mov r6,#4
       mov r0,#disp_mem
p4_l1:
       mov a,#0ffh
       mov @r0,a      
       inc r0
       djnz r6,p4_l1

       call send_disp
       jmp wdh


prg5:                        ; Display loeschen: alle Segmente aus
       mov r6,#4
       mov r0,#disp_mem
p5_l1:
       clr a
       mov @r0,a
       inc r0
       djnz r6,p5_l1

       call send_disp
       jmp wdh


prg6:                   ; Helligkeit der Segmente steuern

p5_w1:
       jnb ri,p5_w1
       mov a,sbuf
       clr ri
       call check_char
       jnb dec_valid,p5_w1 

       cjne a,#8,p5_n1
p5_n1: 
       jnc p5_w1
       swap a
       orl a,#7
       mov b,a

       call i2c_start
       mov a,#70h
       call i2c_send
       mov a,#0h
       call i2c_send
       mov a,b
       call i2c_send
       call i2c_stop
       jmp wdh



check_char:             ; empfangenes Zeichen ueberpruefen
       clr nibble_valid
       clr hex_valid
       clr dec_valid
       cjne a,#'!',n0   ; ! -> Kommando abbrechen
       pop acc          ; Stack aufraeumen
       pop acc
       jmp wdh
n0:
       cjne a,#' ',n01  ; Leerzeichen
       mov a,#16
       setb dec_valid
       setb hex_valid
       ret
n01:
       cjne a,#'-',n02  ; Minuszeichen -
       mov a,#17
       setb dec_valid
       setb hex_valid
       ret
n02:
       cjne a,#'h',n03  ; h
       mov a,#18
       setb hex_valid
       ret
n03:
       cjne a,#'H',n04  ; H
       mov a,#19
       setb hex_valid
       ret
n04:

       cjne a,#48,n1
n1:
       jc ignore
       cjne a,#58,n2
n2:
       jc hex_ok1
       cjne a,#65,n3
n3:
       jc ignore
       cjne a,#71,n4
n4:
       jc hex_ok2
       cjne a,#97,n5
n5:
       jc ignore
       cjne a,#103,n6
n6:
       jc hex_ok3
       jmp ignore


ignore:            ; ungueltiges Zeichen
       ret

hex_ok1:           ; 0..9
       clr c
       subb a,#48
       setb dec_valid
       setb hex_valid
       setb nibble_valid
       ret

hex_ok2:           ; A..F
       clr c
       subb a,#55
       setb hex_valid
       setb nibble_valid
       ret

hex_ok3:           ; a..f
       clr c
       subb a,#87
       setb hex_valid
       setb nibble_valid
       ret



send_disp:                 ; Daten auf Display schreiben
       call i2c_start      ; I2C-Start
       mov a,#70h          ; Adresse SAA1064
       call i2c_send
       mov a,#1h           ; Instructionbyte SAA1064
       call i2c_send

       mov r6,#4
       mov r0,#(disp_mem+3)
send_loop:
       mov a,@r0
       call i2c_send
       dec r0
       djnz r6,send_loop

       call i2c_stop       ; I2C-Stop
       ret



send_disp_init:            ; Init.-Daten auf Display schreiben
       call i2c_start      ; I2C-Start
       mov a,#70h          ; Adresse SAA1064
       call i2c_send
       mov a,#0h           ; Instructionbyte SAA1064
       call i2c_send
       mov a,#37h          ; Controlbyte
       call i2c_send

       mov r6,#4
send_loop_i:
       clr a
       call i2c_send
       djnz r6,send_loop_i

       call i2c_stop       ; I2C-Stop
       ret


init_disp:                        ; Initialisierung: alle Segmente aus
       mov r6,#4
       mov r0,#disp_mem
init_l1:
       clr a
       mov @r0,a      
       inc r0
       djnz r6,init_l1

       call send_disp_init
       ret




i2c_start:                 ; I2C Startbedingung
       setb sda
       nop
       setb scl
       call i2c_wait

       clr sda             ; Data auf 0 (Start)
       nop
       clr scl             ; Clock auf 0
       ret


i2c_stop:                  ; I2C Stopbedingung
       setb scl            ; Clock auf 1
       nop
       setb sda            ; Data auf 1 (Stop)
       ret



i2c_send:                  ; 1 Datenbyte (Akku) ueber I2C senden
       mov r3,#8
i2c_sloop:
       rlc a
       mov sda,c
       nop
       setb scl
       call i2c_wait
       clr scl
       djnz r3,i2c_sloop

       nop
       setb sda
       nop
       setb scl
       call i2c_wait           
       mov c,sda            ; Acknowledge-Bit lesen
       mov i2c_ack_in,c
       clr scl
       nop
       clr sda
       ret



i2c_read:                   ; 1 Datenbyte ueber I2C empfangen (Akku)
       mov r3,#8
       clr a
       setb sda
i2c_rloop:
       call i2c_wait
       setb scl
       nop
       mov c,sda
       rlc a
       nop
       clr scl
       djnz r3,i2c_rloop

       mov c,i2c_ack_out   ; Acknowledge-Bit senden
       mov sda,c
       nop
       setb scl
       call i2c_wait
       clr scl
       nop
       clr sda
       ret


i2c_wait:               ; Verzoegerung fuer I2C (CALL + RET -> 4us bei 12MHz)
       ret



disp_tab:            ; Code-Tabelle fuer Displayzeichen
;          0   1   2   3   4   5   6   7   8   9   A   b   C   d   E   F
       db 3Fh,06h,5Bh,4Fh,66h,6Dh,7Dh,07h,7Fh,6Fh,77h,7Ch,39h,5Eh,79h,71h

;         Blank  -   h   H
       db  00h, 40h,74h,76h


       END
